unit MainF;

interface

uses
  Windows, Dialogs, Messages, SysUtils, Forms, IdHash, IdHashMessageDigest,
  BTAES, Base64U, ShellAPI, Menus, ExtCtrls, StdCtrls, Classes, Controls;

const
  CMinLength       = 128;
  CSystemTrayEvent = WM_USER + 12121;

type
  TMainForm = class(TForm)
    MainPopupMenu : TPopupMenu;
    CloseMenuItem: TMenuItem;
    NoMenuItem: TMenuItem;
    HideMenuItem: TMenuItem;
    PasswordLabel: TLabel;
    PasswordEdit: TEdit;
    InformationLabel: TLabel;
    InformationEdit: TEdit;
    CalculateButton: TButton;
    SpecialCheckBox: TCheckBox;
    NoBevel: TBevel;
    ResultEdit: TEdit;
    CopyrightLabel: TLabel;
    AgainLabel: TLabel;
    AgainEdit: TEdit;

    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure CloseMenuItemClick(Sender: TObject);
    procedure PasswordEditKeyPress(Sender: TObject; var Key: Char);
    procedure InformationEditKeyPress(Sender: TObject; var Key: Char);
    procedure SpecialCheckBoxKeyPress(Sender: TObject; var Key: Char);
    procedure CalculateButtonClick(Sender: TObject);
    procedure ResultEditKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure AgainEditKeyPress(Sender: TObject; var Key: Char);
    procedure ResultEditKeyPress(Sender: TObject; var Key: Char);
  private
    { Private-Deklarationen }
    FCanClose : Boolean;
    FEncrypt  : TBTAES128;
    FHash     : TIdHashMessageDigest5;

    procedure HandleEvent(var AMessage : TMessage); message CSystemTrayEvent;
    procedure PutToSystemTray(ATip : String);
    procedure TakeFromSystemTray;
  public
    { Public-Deklarationen }
  end;

var
  MainForm : TMainForm;

implementation

{$R *.dfm}

procedure TMainForm.HandleEvent(var AMessage : TMessage);
var
  LMousePos : TPoint;
begin
  if (AMessage.Msg = CSystemTrayEvent) then
  begin
    case AMessage.LParamLo of
      WM_LBUTTONDBLCLK :
      begin
        TakeFromSystemTray;
        Application.ShowMainForm := true;
        Show;
        FormStyle := fsStayOnTop;
        BringToFront;

        PasswordEdit.SetFocus;
      end;

      WM_RBUTTONDOWN :
      begin
        GetCursorPos(LMousePos);
        MainPopupMenu.Popup(LMousePos.X, LMousePos.Y);
      end;
    end;
  end;
end;

procedure TMainForm.PutToSystemTray(ATip : String);
var
  LNotifyIconData : TNotifyIconData;
begin
  try
    FillChar(LNotifyIconData.szTip, Length(LNotifyIconData.szTip), 0);
    if (Length(ATip) < Pred(Length(LNotifyIconData.szTip))) then
      StrLCopy(@LNotifyIconData.szTip[0], @ATip[1], Length(ATip))
    else
      StrLCopy(@LNotifyIconData.szTip[0], @ATip[1], Pred(Length(LNotifyIconData.szTip)));

    LNotifyIconData.cbSize           := SizeOf(TNotifyIconData);
    LNotifyIconData.hIcon            := Icon.Handle;
    LNotifyIconData.uCallbackMessage := CSystemTrayEvent;
    LNotifyIconData.uFlags           := NIF_MESSAGE or NIF_ICON or NIF_TIP;
    LNotifyIconData.uID              := 1;
    LNotifyIconData.Wnd              := Handle;

    Shell_NotifyIcon(NIM_ADD, @LNotifyIconData);
  except
  end;
end;

procedure TMainForm.TakeFromSystemTray;
var
  LNotifyIconData : TNotifyIconData;
begin
  try
    LNotifyIconData.cbSize := SizeOf(TNotifyIconData);
    LNotifyIconData.uID    := 1;
    LNotifyIconData.Wnd    := Handle;

    Shell_NotifyIcon(NIM_DELETE, @LNotifyIconData);
  except
  end;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  FCanClose := false;
  FEncrypt  := TBTAES128.Create;
  FHash     := TIdHashMessageDigest5.Create;

  FormStyle := fsNormal;
  Hide;
  Application.ShowMainForm := false;
  PutToSystemTray(Caption);
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if not(FCanClose) then
  begin
    FormStyle := fsNormal;
    Hide;
    Application.ShowMainForm := false;
    PutToSystemTray(Caption);

    Action := caNone;
  end;
end;

procedure TMainForm.CloseMenuItemClick(Sender: TObject);
begin
  FCanClose := true;
  TakeFromSystemTray;
  Close;
end;

procedure TMainForm.PasswordEditKeyPress(Sender: TObject; var Key: Char);
begin
  if (Key = #13) then
  begin
    AgainEdit.SetFocus;
    Key := #0;
  end;
end;

procedure TMainForm.InformationEditKeyPress(Sender: TObject; var Key: Char);
begin
  if (Key = #13) then
  begin
    SpecialCheckBox.SetFocus;
    Key := #0;
  end;
end;

procedure TMainForm.SpecialCheckBoxKeyPress(Sender: TObject; var Key: Char);
begin
  if (Key = #13) then
  begin
    CalculateButton.SetFocus;
    Key := #0;
  end;
end;

procedure TMainForm.CalculateButtonClick(Sender: TObject);
  function HashToString(AHash : T4x4LongWordRecord) : String;
  var
    LIndex   : Byte;
    LPointer : PChar;
  begin
    SetLength(Result, Length(AHash) * SizeOf(LongWord));

    LPointer := PChar(@AHash);
    for LIndex := 0 to Pred(Length(Result)) do
      Result[Succ(LIndex)] := LPointer[LIndex];
  end;
var
  LChar        : Char;
  LInBytes     : array of Byte;
  LIndex       : Byte;
  LInformation : String;
  LPassword    : String;
  LOutBytes    : array of Byte;
  LTemp        : String;
begin
  if (PasswordEdit.Text = AgainEdit.Text) then
  begin
    LInformation := '';
    LTemp        := HashToString(FHash.HashValue(EncBase64Encode(InformationEdit.Text)));
    while (Length(LInformation) < CMinLength) do
      LInformation := LInformation + LTemp;

    LPassword := '';
    LTemp     := HashToString(FHash.HashValue(EncBase64Encode(PasswordEdit.Text)));
    while (Length(LPassword) < CMinLength) do
      LPassword := LPassword + LTemp;

    SetLength(LInBytes,  Length(LInformation));
    SetLength(LOutBytes, Length(LInformation));
    for LIndex := 1 to Length(LInformation) do
      LInBytes[Pred(LIndex)] := Ord(LInformation[LIndex]);
    FEncrypt.encrypt(LPassword, LInBytes, LOutBytes);
    for LIndex := 1 to Length(LInformation) do
      LInformation[LIndex] := Chr(LOutBytes[Pred(LIndex)]);
    LInformation := EncBase64Encode(LInformation);

    for LIndex := 1 to Length(LInformation) do
    begin
      LChar := LInformation[LIndex];

      if (SpecialCheckBox.Checked) then
      begin
        if ((LIndex mod 3) = 0) then
        begin
          case LChar of
            '0'..'9' : LChar := '!';
            'A'..'Z' : LChar := '*';
            'a'..'z' : LChar := '$';
          end;
        end;
      end
      else
      begin
        if (LChar in ['+', '/']) then
          LChar := '0';
      end;

      LInformation[LIndex] := LChar;
    end;

    InformationEdit.Text    := '';
    SpecialCheckBox.Checked := true;

    ResultEdit.Text := LInformation;
    ResultEdit.SetFocus;
    ResultEdit.SelectAll;
  end
  else
    ShowMessage('Passwrter stimmen nicht berein!');
end;

procedure TMainForm.ResultEditKeyUp(Sender: TObject; var Key: Word; Shift : TShiftState);
begin
  if (Shift = [ssCtrl]) then
  begin
    if (Key = Ord('c')) then
      ResultEdit.CopyToClipboard;
    if (Key = Ord('v')) then
      ResultEdit.PasteFromClipboard;
    if (Key = Ord('x')) then
      ResultEdit.CutToClipboard;
  end;
end;

procedure TMainForm.AgainEditKeyPress(Sender: TObject; var Key: Char);
begin
  if (Key = #13) then
  begin
    InformationEdit.SetFocus;
    Key := #0;
  end;
end;

procedure TMainForm.ResultEditKeyPress(Sender: TObject; var Key: Char);
begin
  if (Key = #13) then
  begin
    InformationEdit.SetFocus;
    Key := #0;
  end;
end;

end.
